#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define RC5_POWER 12
#define RC5_VOLUME_UP 16
#define RC5_VOLUME_DOWN 17

volatile uint8_t bitNumber = 0;
volatile uint16_t packet = 0;
volatile uint8_t newPacket = 0;
volatile uint8_t firstHalf = 0;

int main(void) {
	DDRB = 0xff;
	TCCR1B = _BV(CS12) | _BV(WGM12);
	OCR1A = 3125;
	TIMSK = _BV(TICIE1) | _BV(OCIE1A);
	sei();
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_enable();
	int8_t ledPointPosition = 0;
	uint8_t ledPointOn = 0;
	uint8_t lastToggle = 2;
	while (1) {
		sleep_cpu();
		if(newPacket){
			newPacket = 0;
			uint8_t command = packet & 0b00111111;
			uint8_t toggle = (packet >> 11) & 1;
			if (command == RC5_VOLUME_DOWN) {
				ledPointPosition++;
				if (ledPointPosition > 7) ledPointPosition = 0;
			}
			if (command == RC5_VOLUME_UP) {
				ledPointPosition--;
				if (ledPointPosition < 0) ledPointPosition = 7;
			}
			if (command == RC5_POWER) {
				if (toggle != lastToggle) {
					ledPointOn = !ledPointOn;
					lastToggle = toggle;	
				}
			}
			if (ledPointOn) {
				PORTB = _BV(ledPointPosition);
			} else {
				PORTB = 0;
			}
		}
	}
}

ISR(TIMER1_CAPT_vect) {
	TCNT1 = 0;
	if (bitNumber) {
		if (ICR1 < 83) firstHalf = !firstHalf;
	}
	if (!firstHalf) {
		packet <<= 1;
		if (!(TCCR1B & _BV(ICES1))) {
			packet |= 1;
		}
		bitNumber++;
		if (bitNumber == 14) newPacket = 1;
	}
	TCCR1B ^= _BV(ICES1);
}

ISR(TIMER1_COMPA_vect) {
	TCCR1B &= ~_BV(ICES1);
	bitNumber = 0;
	packet = 0;
	firstHalf = 0;
}
